@@ -14,13 +14,22 @@ import open3d.visualization.rendering as rendering
 
 from .methods import IntParameter, EnumParameter, StereoOutput, StereoMethod, Calibration, InputPair
    
-def show_color_disparity (name: str, disparity_map: np.ndarray):
-    min_disp = 0
-    max_disp = 64
+disparity_window = None
+
+def color_disparity (disparity_map: np.ndarray, calibration: Calibration):
+    min_disp = (calibration.fx * calibration.baseline_meters) / calibration.depth_range[1]
+    # disparity_pixels = (calibration.fx * calibration.baseline_meters) / depth_meters
+    max_disp = (calibration.fx * calibration.baseline_meters) / calibration.depth_range[0]
     norm_disparity_map = 255*((disparity_map-min_disp) / (max_disp-min_disp))
-    disparity_color = cv2.applyColorMap(cv2.convertScaleAbs(norm_disparity_map, 1), cv2.COLORMAP_MAGMA)
-    cv2.namedWindow (name, cv2.WINDOW_KEEPRATIO)
-    cv2.imshow (name, disparity_color)
+    disparity_color = cv2.applyColorMap(cv2.convertScaleAbs(norm_disparity_map, 1), cv2.COLORMAP_VIRIDIS)
+    return disparity_color
+
+def show_color_disparity (name: str, color_disparity: np.ndarray):
+    global disparity_window
+    if disparity_window is None:
+        disparity_window = cv2.namedWindow ("Disparity", cv2.WINDOW_KEEPRATIO)
+    cv2.imshow ("Disparity", color_disparity)
+    cv2.setWindowTitle ("Disparity", name)
 
 class Settings:
     def __init__(self):
@@ -30,10 +39,23 @@ class Source:
     def __init__(self):
         pass
 
+    @abstractmethod
+    def is_live(self) -> bool:
+        """Whether the source is capture live images or not"""
+        return False
+
+    def selected_index (self) -> int:
+        return 0
+
     @abstractmethod
     def get_next_pair(self) -> InputPair:
         return InputPair(None, None, None, None)
 
+    def get_pair_at_index(self, idx: int) -> InputPair:
+        return InputPair(None, None, None, None)
+
+    def get_pair_list(self) -> List[str]:
+        return []
 
 class Visualizer:
     def __init__(self, stereo_methods: Dict[str, StereoMethod], source: Source):
@@ -79,6 +101,33 @@ class Visualizer:
         self._next_image_button = gui.Button("Next Image")
         self._next_image_button.set_on_clicked(self._next_image_clicked)
         self._settings_panel.add_child(self._next_image_button)
+        if not self.source.is_live():
+            # self._next_image_button = gui.Button("Next")
+            # self._next_image_button.set_on_clicked(self._next_image_clicked)
+            # horiz.add_child(self._next_image_button)
+
+            # self.images_combo = gui.ListView()
+            # input_pairs = self.source.get_pair_list()
+            # self.images_combo.set_items(input_pairs)
+            # self.images_combo.selected_index = 0
+            # self.images_combo.set_max_visible_items(3)
+            # self.images_combo.set_on_selection_changed(self._image_selected)
+            # self.images_combo.tooltip = self.images_combo.selected_value
+            # self._settings_panel.add_child(self.images_combo)
+            # self._settings_panel.add_fixed(self.separation_height)
+            # horiz.add_child(self.images_combo)
+
+            self._settings_panel.add_fixed(self.separation_height)
+            self.images_combo = gui.Combobox()
+            input_pairs = self.source.get_pair_list()
+            for pair_name in input_pairs:
+                self.images_combo.add_item(pair_name)
+            self.images_combo.selected_index = 0
+            self.images_combo.set_on_selection_changed(self._image_selected)
+            self._settings_panel.add_child(self.images_combo)
+            self._settings_panel.add_fixed(self.separation_height)
+        else:
+            self.images_combo = None
         
         horiz = gui.Horiz(0, gui.Margins(0.25 * em, 0.25 * em, 0.25 * em, 0.25 * em))
         label = gui.Label("Input downsampling")
@@ -113,20 +162,22 @@ class Visualizer:
         reset_cam_button = gui.Button("Reset Camera")
         reset_cam_button.set_on_clicked(self._reset_camera)
         view_ctrls.add_child(reset_cam_button)
-        self._show_axes = gui.Checkbox("Show axes")
-        self._show_axes.set_on_checked(self._on_show_axes)
-        view_ctrls.add_child(self._show_axes)
+        # self._show_axes = gui.Checkbox("Show axes")
+        # self._show_axes.set_on_checked(self._on_show_axes)
+        # view_ctrls.add_child(self._show_axes)
         
         horiz = gui.Horiz(0, gui.Margins(0.25 * em, 0.25 * em, 0.25 * em, 0.25 * em))
         label = gui.Label("Max depth (m)")
         label.tooltip = "Max depth to render in meters"
         horiz.add_child(label)
-        self.depth_range_slider = gui.Slider(gui.Slider.INT)
-        self.depth_range_slider.set_limits(1, 1000)
-        self.depth_range_slider.int_value = 100
-        self.depth_range_slider.set_on_value_changed(lambda v: self._update_rendering())
+        self.depth_range_slider = gui.Slider(gui.Slider.DOUBLE)
+        self.depth_range_slider.set_limits(0.5, 1000)
+        self.depth_range_slider.double_value = 100
+        self.depth_range_slider.set_on_value_changed(self._depth_range_slider_changed)
         horiz.add_child(self.depth_range_slider)
         view_ctrls.add_child(horiz)
+
+        self._depth_range_manually_changed = False
         
         self._settings_panel.add_fixed(self.separation_height)
         self._settings_panel.add_child(view_ctrls)
@@ -138,7 +189,10 @@ class Visualizer:
         self._on_algo_list_selected(self.algo_list.selected_value, False)
         self._apply_settings()
 
-        self.read_next_pair ()
+        if self.source.is_live():
+            self.read_next_pair ()
+        else:
+            self._image_selected (None, None)
 
     def _on_key_pressed (self, keyEvent):
         if keyEvent.key == gui.KeyName.Q:
@@ -162,6 +216,7 @@ class Visualizer:
 
     def read_next_pair (self):
         input = self.source.get_next_pair ()
+        self._update_pair_index ()
         self._process_input (input)
 
     def _process_input (self, input):
@@ -171,6 +226,10 @@ class Visualizer:
             self._downsample_input (input)
         else:
             self.full_res_input = input
+
+        if not self._depth_range_manually_changed:
+            self.depth_range_slider.double_value = input.calibration.depth_range[1]
+
         cv2.namedWindow ("Input image", cv2.WINDOW_KEEPRATIO)
         cv2.imshow ("Input image", np.hstack([input.left_image, input.right_image]))
         self.input = input
@@ -219,6 +278,11 @@ class Visualizer:
         up = np.array([0, 1.0, 0])
         self._scene.look_at(lookat, eye, up)
 
+        if self.input.has_data():
+            self._depth_range_manually_changed = False
+            self.depth_range_slider.double_value = self.input.calibration.depth_range[1]
+            self._update_rendering ()
+
     def _build_stereo_method_widgets(self, name):
         em = self.window.theme.font_size
         method = self.stereo_methods[name]
@@ -287,6 +351,8 @@ class Visualizer:
         self._apply_settings()
         if self.stereo_methods_output[name].disparity_pixels is None:
             self._run_current_method ()
+        if self.stereo_methods_output[name].disparity_color is not None:
+            show_color_disparity (name, self.stereo_methods_output[name].disparity_color)
 
     def _on_show_axes(self, show):
         self.settings.show_axes = show
@@ -295,6 +361,15 @@ class Visualizer:
     def _next_image_clicked(self):
         self.read_next_pair ()
 
+    def _image_selected(self, combo_idx, combo_val):
+        idx = self.images_combo.selected_index
+        input = self.source.get_pair_at_index (idx)
+        self._process_input (input)
+
+    def _update_pair_index (self):
+        if self.images_combo is not None:
+            self.images_combo.selected_index = self.source.selected_index()
+
     def _apply_settings(self):
         self._scene.scene.show_axes(self.settings.show_axes)
 
@@ -322,17 +397,33 @@ class Visualizer:
         stereo_output = self.executor_future.result()
         self.executor_future = None
 
+        x0,y0,x1,y1 = self.input.calibration.left_image_rect_normalized
+        x0 = int(x0*stereo_output.disparity_pixels.shape[1] + 0.5)
+        x1 = int(x1*stereo_output.disparity_pixels.shape[1] + 0.5)
+        y0 = int(y0*stereo_output.disparity_pixels.shape[0] + 0.5)
+        y1 = int(y1*stereo_output.disparity_pixels.shape[0] + 0.5)
+        valid_mask = np.zeros(stereo_output.disparity_pixels.shape, dtype=np.uint8)
+        valid_mask[y0:y1, x0:x1] = 1
+        stereo_output.disparity_pixels[valid_mask == 0] = -1.0
+
         name = self.algo_list.selected_value
-        show_color_disparity (name, stereo_output.disparity_pixels)
+        stereo_output.disparity_color = color_disparity (stereo_output.disparity_pixels, self.input.calibration)
+        show_color_disparity (name, stereo_output.disparity_color)
 
         self.stereo_methods_output[name] = stereo_output
         self._update_rendering ([name])
         self._update_runtime ()
     
+    def _depth_range_slider_changed(self, v: float):
+        self._depth_range_manually_changed = True
+        self._update_rendering()
+
     def _update_rendering (self, names_to_update=None):
         if names_to_update is None:
             names_to_update = list(self.stereo_methods_output.keys())
 
+        selected_name = self.algo_list.selected_value
+
         for name in names_to_update:
             stereo_output = self.stereo_methods_output[name]
             if stereo_output.disparity_pixels is None:
@@ -343,13 +434,15 @@ class Visualizer:
             if self._scene.scene.has_geometry(name):
                 self._scene.scene.remove_geometry(name)
 
+            self._scene.scene.show_geometry(name, name == selected_name)
+
             o3d_color = o3d.geometry.Image(cv2.cvtColor(stereo_output.color_image_bgr, cv2.COLOR_BGR2RGB))
             o3d_depth = o3d.geometry.Image(depth_meters)
             rgbd = o3d.geometry.RGBDImage.create_from_color_and_depth(o3d_color,
-                                                                    o3d_depth,
-                                                                    1,
-                                                                    depth_trunc=self.depth_range_slider.int_value,
-                                                                    convert_rgb_to_intensity=False)
+                                                                      o3d_depth,
+                                                                      1,
+                                                                      depth_trunc=self.depth_range_slider.int_value,
+                                                                      convert_rgb_to_intensity=False)
             stereo_output.point_cloud = o3d.geometry.PointCloud.create_from_rgbd_image(rgbd, self.o3dCameraIntrinsic)
             stereo_output.point_cloud.transform([[1,0,0,0],[0,-1,0,0],[0,0,-1,0],[0,0,0,1]])
             self._scene.scene.add_geometry(name, stereo_output.point_cloud, rendering.MaterialRecord())
